@@ -1,5 +1,5 @@
 # engine/reflection.py
-# Copyright (C) 2005-2016 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2020 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -25,29 +25,27 @@ methods such as get_table_names, get_columns, etc.
    'name' attribute..
 """
 
-from .. import exc, sql
-from ..sql import schema as sa_schema
+from .base import Connectable
+from .. import exc
+from .. import inspection
+from .. import sql
 from .. import util
+from ..sql import operators
+from ..sql import schema as sa_schema
 from ..sql.type_api import TypeEngine
 from ..util import deprecated
 from ..util import topological
-from .. import inspection
-from .base import Connectable
 
 
 @util.decorator
 def cache(fn, self, con, *args, **kw):
-    info_cache = kw.get('info_cache', None)
+    info_cache = kw.get("info_cache", None)
     if info_cache is None:
         return fn(self, con, *args, **kw)
     key = (
         fn.__name__,
         tuple(a for a in args if isinstance(a, util.string_types)),
-        tuple((k, v) for k, v in kw.items() if
-              isinstance(v,
-                         util.string_types + util.int_types + (float, )
-                         )
-              )
+        tuple((k, v) for k, v in kw.items() if k != "info_cache"),
     )
     ret = info_cache.get(key)
     if ret is None:
@@ -99,7 +97,7 @@ class Inspector(object):
         self.bind = bind
 
         # set the engine
-        if hasattr(bind, 'engine'):
+        if hasattr(bind, "engine"):
             self.engine = bind.engine
         else:
             self.engine = bind
@@ -130,7 +128,7 @@ class Inspector(object):
         See the example at :class:`.Inspector`.
 
         """
-        if hasattr(bind.dialect, 'inspector'):
+        if hasattr(bind.dialect, "inspector"):
             return bind.dialect.inspector(bind)
         return Inspector(bind)
 
@@ -153,11 +151,22 @@ class Inspector(object):
         """Return all schema names.
         """
 
-        if hasattr(self.dialect, 'get_schema_names'):
-            return self.dialect.get_schema_names(self.bind,
-                                                 info_cache=self.info_cache)
+        if hasattr(self.dialect, "get_schema_names"):
+            return self.dialect.get_schema_names(
+                self.bind, info_cache=self.info_cache
+            )
         return []
 
+    @util.deprecated_params(
+        order_by=(
+            "1.0",
+            "The :paramref:`get_table_names.order_by` parameter is deprecated "
+            "and will be removed in a future release.  Please refer to "
+            ":meth:`.Inspector.get_sorted_table_and_fkc_names` for a "
+            "more comprehensive solution to resolving foreign key cycles "
+            "between tables.",
+        )
+    )
     def get_table_names(self, schema=None, order_by=None):
         """Return all table names in referred to within a particular schema.
 
@@ -177,17 +186,6 @@ class Inspector(object):
          resolve cycles, and will raise :class:`.CircularDependencyError`
          if cycles exist.
 
-         .. deprecated:: 1.0.0 - see
-            :meth:`.Inspector.get_sorted_table_and_fkc_names` for a version
-            of this which resolves foreign key cycles between tables
-            automatically.
-
-         .. versionchanged:: 0.8 the "foreign_key" sorting sorts tables
-            in order of dependee to dependent; that is, in creation
-            order, rather than in drop order.  This is to maintain
-            consistency with similar features such as
-            :attr:`.MetaData.sorted_tables` and :func:`.util.sort_tables`.
-
         .. seealso::
 
             :meth:`.Inspector.get_sorted_table_and_fkc_names`
@@ -196,17 +194,18 @@ class Inspector(object):
 
         """
 
-        if hasattr(self.dialect, 'get_table_names'):
+        if hasattr(self.dialect, "get_table_names"):
             tnames = self.dialect.get_table_names(
-                self.bind, schema, info_cache=self.info_cache)
+                self.bind, schema, info_cache=self.info_cache
+            )
         else:
             tnames = self.engine.table_names(schema)
-        if order_by == 'foreign_key':
+        if order_by == "foreign_key":
             tuples = []
             for tname in tnames:
                 for fkey in self.get_foreign_keys(tname, schema):
-                    if tname != fkey['referred_table']:
-                        tuples.append((fkey['referred_table'], tname))
+                    if tname != fkey["referred_table"]:
+                        tuples.append((fkey["referred_table"], tname))
             tnames = list(topological.sort(tuples, tnames))
         return tnames
 
@@ -234,9 +233,10 @@ class Inspector(object):
              with an already-given :class:`.MetaData`.
 
         """
-        if hasattr(self.dialect, 'get_table_names'):
+        if hasattr(self.dialect, "get_table_names"):
             tnames = self.dialect.get_table_names(
-                self.bind, schema, info_cache=self.info_cache)
+                self.bind, schema, info_cache=self.info_cache
+            )
         else:
             tnames = self.engine.table_names(schema)
 
@@ -246,20 +246,17 @@ class Inspector(object):
         fknames_for_table = {}
         for tname in tnames:
             fkeys = self.get_foreign_keys(tname, schema)
-            fknames_for_table[tname] = set(
-                [fk['name'] for fk in fkeys]
-            )
+            fknames_for_table[tname] = set([fk["name"] for fk in fkeys])
             for fkey in fkeys:
-                if tname != fkey['referred_table']:
-                    tuples.add((fkey['referred_table'], tname))
+                if tname != fkey["referred_table"]:
+                    tuples.add((fkey["referred_table"], tname))
         try:
             candidate_sort = list(topological.sort(tuples, tnames))
         except exc.CircularDependencyError as err:
             for edge in err.edges:
                 tuples.remove(edge)
                 remaining_fkcs.update(
-                    (edge[1], fkc)
-                    for fkc in fknames_for_table[edge[1]]
+                    (edge[1], fkc) for fkc in fknames_for_table[edge[1]]
                 )
 
             candidate_sort = list(topological.sort(tuples, tnames))
@@ -278,7 +275,8 @@ class Inspector(object):
 
         """
         return self.dialect.get_temp_table_names(
-            self.bind, info_cache=self.info_cache)
+            self.bind, info_cache=self.info_cache
+        )
 
     def get_temp_view_names(self):
         """return a list of temporary view names for the current bind.
@@ -290,7 +288,8 @@ class Inspector(object):
 
         """
         return self.dialect.get_temp_view_names(
-            self.bind, info_cache=self.info_cache)
+            self.bind, info_cache=self.info_cache
+        )
 
     def get_table_options(self, table_name, schema=None, **kw):
         """Return a dictionary of options specified when the table of the
@@ -306,10 +305,10 @@ class Inspector(object):
          use :class:`.quoted_name`.
 
         """
-        if hasattr(self.dialect, 'get_table_options'):
+        if hasattr(self.dialect, "get_table_options"):
             return self.dialect.get_table_options(
-                self.bind, table_name, schema,
-                info_cache=self.info_cache, **kw)
+                self.bind, table_name, schema, info_cache=self.info_cache, **kw
+            )
         return {}
 
     def get_view_names(self, schema=None):
@@ -320,8 +319,9 @@ class Inspector(object):
 
         """
 
-        return self.dialect.get_view_names(self.bind, schema,
-                                           info_cache=self.info_cache)
+        return self.dialect.get_view_names(
+            self.bind, schema, info_cache=self.info_cache
+        )
 
     def get_view_definition(self, view_name, schema=None):
         """Return definition for `view_name`.
@@ -332,7 +332,8 @@ class Inspector(object):
         """
 
         return self.dialect.get_view_definition(
-            self.bind, view_name, schema, info_cache=self.info_cache)
+            self.bind, view_name, schema, info_cache=self.info_cache
+        )
 
     def get_columns(self, table_name, schema=None, **kw):
         """Return information about columns in `table_name`.
@@ -340,20 +341,17 @@ class Inspector(object):
         Given a string `table_name` and an optional string `schema`, return
         column information as a list of dicts with these keys:
 
-        name
-          the column's name
+        * ``name`` - the column's name
 
-        type
+        * ``type`` - the type of this column; an instance of
           :class:`~sqlalchemy.types.TypeEngine`
 
-        nullable
-          boolean
+        * ``nullable`` - boolean flag if the column is NULL or NOT NULL
 
-        default
-          the column's default value
+        * ``default`` - the column's server default value - this is returned
+          as a string SQL expression.
 
-        attrs
-          dict containing optional column attributes
+        * ``attrs``  - dict containing optional column attributes
 
         :param table_name: string name of the table.  For special quoting,
          use :class:`.quoted_name`.
@@ -362,20 +360,27 @@ class Inspector(object):
          of the database connection.  For special quoting,
          use :class:`.quoted_name`.
 
+        :return: list of dictionaries, each representing the definition of
+         a database column.
+
         """
 
-        col_defs = self.dialect.get_columns(self.bind, table_name, schema,
-                                            info_cache=self.info_cache,
-                                            **kw)
+        col_defs = self.dialect.get_columns(
+            self.bind, table_name, schema, info_cache=self.info_cache, **kw
+        )
         for col_def in col_defs:
             # make this easy and only return instances for coltype
-            coltype = col_def['type']
+            coltype = col_def["type"]
             if not isinstance(coltype, TypeEngine):
-                col_def['type'] = coltype()
+                col_def["type"] = coltype()
         return col_defs
 
-    @deprecated('0.7', 'Call to deprecated method get_primary_keys.'
-                '  Use get_pk_constraint instead.')
+    @deprecated(
+        "0.7",
+        "The :meth:`.Inspector.get_primary_keys` method is deprecated and "
+        "will be removed in a future release.  Please refer to the "
+        ":meth:`.Inspector.get_pk_constraint` method.",
+    )
     def get_primary_keys(self, table_name, schema=None, **kw):
         """Return information about primary keys in `table_name`.
 
@@ -383,9 +388,9 @@ class Inspector(object):
         primary key information as a list of column names.
         """
 
-        return self.dialect.get_pk_constraint(self.bind, table_name, schema,
-                                              info_cache=self.info_cache,
-                                              **kw)['constrained_columns']
+        return self.dialect.get_pk_constraint(
+            self.bind, table_name, schema, info_cache=self.info_cache, **kw
+        )["constrained_columns"]
 
     def get_pk_constraint(self, table_name, schema=None, **kw):
         """Return information about primary key constraint on `table_name`.
@@ -407,9 +412,9 @@ class Inspector(object):
          use :class:`.quoted_name`.
 
         """
-        return self.dialect.get_pk_constraint(self.bind, table_name, schema,
-                                              info_cache=self.info_cache,
-                                              **kw)
+        return self.dialect.get_pk_constraint(
+            self.bind, table_name, schema, info_cache=self.info_cache, **kw
+        )
 
     def get_foreign_keys(self, table_name, schema=None, **kw):
         """Return information about foreign_keys in `table_name`.
@@ -442,9 +447,9 @@ class Inspector(object):
 
         """
 
-        return self.dialect.get_foreign_keys(self.bind, table_name, schema,
-                                             info_cache=self.info_cache,
-                                             **kw)
+        return self.dialect.get_foreign_keys(
+            self.bind, table_name, schema, info_cache=self.info_cache, **kw
+        )
 
     def get_indexes(self, table_name, schema=None, **kw):
         """Return information about indexes in `table_name`.
@@ -461,6 +466,12 @@ class Inspector(object):
         unique
           boolean
 
+        column_sorting
+          optional dict mapping column names to tuple of sort keywords,
+          which may include ``asc``, ``desc``, ``nullsfirst``, ``nullslast``.
+
+          .. versionadded:: 1.3.5
+
         dialect_options
           dict of dialect-specific index options.  May not be present
           for all dialects.
@@ -476,9 +487,9 @@ class Inspector(object):
 
         """
 
-        return self.dialect.get_indexes(self.bind, table_name,
-                                        schema,
-                                        info_cache=self.info_cache, **kw)
+        return self.dialect.get_indexes(
+            self.bind, table_name, schema, info_cache=self.info_cache, **kw
+        )
 
     def get_unique_constraints(self, table_name, schema=None, **kw):
         """Return information about unique constraints in `table_name`.
@@ -499,12 +510,31 @@ class Inspector(object):
          of the database connection.  For special quoting,
          use :class:`.quoted_name`.
 
-        .. versionadded:: 0.8.4
-
         """
 
         return self.dialect.get_unique_constraints(
-            self.bind, table_name, schema, info_cache=self.info_cache, **kw)
+            self.bind, table_name, schema, info_cache=self.info_cache, **kw
+        )
+
+    def get_table_comment(self, table_name, schema=None, **kw):
+        """Return information about the table comment for ``table_name``.
+
+        Given a string ``table_name`` and an optional string ``schema``,
+        return table comment information as a dictionary with these keys:
+
+        text
+            text of the comment.
+
+        Raises ``NotImplementedError`` for a dialect that does not support
+        comments.
+
+        .. versionadded:: 1.2
+
+        """
+
+        return self.dialect.get_table_comment(
+            self.bind, table_name, schema, info_cache=self.info_cache, **kw
+        )
 
     def get_check_constraints(self, table_name, schema=None, **kw):
         """Return information about check constraints in `table_name`.
@@ -518,6 +548,12 @@ class Inspector(object):
         sqltext
           the check constraint's SQL expression
 
+        dialect_options
+          may or may not be present; a dictionary with additional
+          dialect-specific options for this CHECK constraint
+
+          .. versionadded:: 1.3.8
+
         :param table_name: string name of the table.  For special quoting,
          use :class:`.quoted_name`.
 
@@ -530,9 +566,17 @@ class Inspector(object):
         """
 
         return self.dialect.get_check_constraints(
-            self.bind, table_name, schema, info_cache=self.info_cache, **kw)
+            self.bind, table_name, schema, info_cache=self.info_cache, **kw
+        )
 
-    def reflecttable(self, table, include_columns, exclude_columns=()):
+    def reflecttable(
+        self,
+        table,
+        include_columns,
+        exclude_columns=(),
+        resolve_fks=True,
+        _extend_on=None,
+    ):
         """Given a Table object, load its internal constructs based on
         introspection.
 
@@ -540,7 +584,7 @@ class Inspector(object):
         table reflection.  Direct usage is like::
 
             from sqlalchemy import create_engine, MetaData, Table
-            from sqlalchemy.engine import reflection
+            from sqlalchemy.engine.reflection import Inspector
 
             engine = create_engine('...')
             meta = MetaData()
@@ -553,6 +597,13 @@ class Inspector(object):
           in the reflection process.  If ``None``, all columns are reflected.
 
         """
+
+        if _extend_on is not None:
+            if table in _extend_on:
+                return
+            else:
+                _extend_on.add(table)
+
         dialect = self.bind.dialect
 
         schema = self.bind.schema_for_object(table)
@@ -571,7 +622,8 @@ class Inspector(object):
 
         # reflect table options, like mysql_engine
         tbl_opts = self.get_table_options(
-            table_name, schema, **table.dialect_kwargs)
+            table_name, schema, **table.dialect_kwargs
+        )
         if tbl_opts:
             # add additional kwargs to the Table if the dialect
             # returned them
@@ -587,174 +639,263 @@ class Inspector(object):
         cols_by_orig_name = {}
 
         for col_d in self.get_columns(
-                table_name, schema, **table.dialect_kwargs):
+            table_name, schema, **table.dialect_kwargs
+        ):
             found_table = True
 
             self._reflect_column(
-                table, col_d, include_columns,
-                exclude_columns, cols_by_orig_name)
+                table,
+                col_d,
+                include_columns,
+                exclude_columns,
+                cols_by_orig_name,
+            )
 
         if not found_table:
             raise exc.NoSuchTableError(table.name)
 
         self._reflect_pk(
-            table_name, schema, table, cols_by_orig_name, exclude_columns)
+            table_name, schema, table, cols_by_orig_name, exclude_columns
+        )
 
         self._reflect_fk(
-            table_name, schema, table, cols_by_orig_name,
-            exclude_columns, reflection_options)
+            table_name,
+            schema,
+            table,
+            cols_by_orig_name,
+            exclude_columns,
+            resolve_fks,
+            _extend_on,
+            reflection_options,
+        )
 
         self._reflect_indexes(
-            table_name, schema, table, cols_by_orig_name,
-            include_columns, exclude_columns, reflection_options)
+            table_name,
+            schema,
+            table,
+            cols_by_orig_name,
+            include_columns,
+            exclude_columns,
+            reflection_options,
+        )
 
         self._reflect_unique_constraints(
-            table_name, schema, table, cols_by_orig_name,
-            include_columns, exclude_columns, reflection_options)
+            table_name,
+            schema,
+            table,
+            cols_by_orig_name,
+            include_columns,
+            exclude_columns,
+            reflection_options,
+        )
 
         self._reflect_check_constraints(
-            table_name, schema, table, cols_by_orig_name,
-            include_columns, exclude_columns, reflection_options)
+            table_name,
+            schema,
+            table,
+            cols_by_orig_name,
+            include_columns,
+            exclude_columns,
+            reflection_options,
+        )
+
+        self._reflect_table_comment(
+            table_name, schema, table, reflection_options
+        )
 
     def _reflect_column(
-        self, table, col_d, include_columns,
-            exclude_columns, cols_by_orig_name):
+        self, table, col_d, include_columns, exclude_columns, cols_by_orig_name
+    ):
 
-        orig_name = col_d['name']
+        orig_name = col_d["name"]
 
         table.dispatch.column_reflect(self, table, col_d)
 
         # fetch name again as column_reflect is allowed to
         # change it
-        name = col_d['name']
-        if (include_columns and name not in include_columns) \
-                or (exclude_columns and name in exclude_columns):
+        name = col_d["name"]
+        if (include_columns and name not in include_columns) or (
+            exclude_columns and name in exclude_columns
+        ):
             return
 
-        coltype = col_d['type']
+        coltype = col_d["type"]
 
         col_kw = dict(
             (k, col_d[k])
-            for k in ['nullable', 'autoincrement', 'quote', 'info', 'key']
+            for k in [
+                "nullable",
+                "autoincrement",
+                "quote",
+                "info",
+                "key",
+                "comment",
+            ]
             if k in col_d
         )
 
-        colargs = []
-        if col_d.get('default') is not None:
-            # the "default" value is assumed to be a literal SQL
-            # expression, so is wrapped in text() so that no quoting
-            # occurs on re-issuance.
-            colargs.append(
-                sa_schema.DefaultClause(
-                    sql.text(col_d['default']), _reflected=True
-                )
-            )
+        if "dialect_options" in col_d:
+            col_kw.update(col_d["dialect_options"])
 
-        if 'sequence' in col_d:
+        colargs = []
+        if col_d.get("default") is not None:
+            default = col_d["default"]
+            if isinstance(default, sql.elements.TextClause):
+                default = sa_schema.DefaultClause(default, _reflected=True)
+            elif not isinstance(default, sa_schema.FetchedValue):
+                default = sa_schema.DefaultClause(
+                    sql.text(col_d["default"]), _reflected=True
+                )
+
+            colargs.append(default)
+
+        if "sequence" in col_d:
             self._reflect_col_sequence(col_d, colargs)
 
-        cols_by_orig_name[orig_name] = col = \
-            sa_schema.Column(name, coltype, *colargs, **col_kw)
+        cols_by_orig_name[orig_name] = col = sa_schema.Column(
+            name, coltype, *colargs, **col_kw
+        )
 
         if col.key in table.primary_key:
             col.primary_key = True
         table.append_column(col)
 
     def _reflect_col_sequence(self, col_d, colargs):
-        if 'sequence' in col_d:
+        if "sequence" in col_d:
             # TODO: mssql and sybase are using this.
-            seq = col_d['sequence']
-            sequence = sa_schema.Sequence(seq['name'], 1, 1)
-            if 'start' in seq:
-                sequence.start = seq['start']
-            if 'increment' in seq:
-                sequence.increment = seq['increment']
+            seq = col_d["sequence"]
+            sequence = sa_schema.Sequence(seq["name"], 1, 1)
+            if "start" in seq:
+                sequence.start = seq["start"]
+            if "increment" in seq:
+                sequence.increment = seq["increment"]
             colargs.append(sequence)
 
     def _reflect_pk(
-            self, table_name, schema, table,
-            cols_by_orig_name, exclude_columns):
+        self, table_name, schema, table, cols_by_orig_name, exclude_columns
+    ):
         pk_cons = self.get_pk_constraint(
-            table_name, schema, **table.dialect_kwargs)
+            table_name, schema, **table.dialect_kwargs
+        )
         if pk_cons:
             pk_cols = [
                 cols_by_orig_name[pk]
-                for pk in pk_cons['constrained_columns']
+                for pk in pk_cons["constrained_columns"]
                 if pk in cols_by_orig_name and pk not in exclude_columns
             ]
 
             # update pk constraint name
-            table.primary_key.name = pk_cons.get('name')
+            table.primary_key.name = pk_cons.get("name")
 
             # tell the PKConstraint to re-initialize
             # its column collection
             table.primary_key._reload(pk_cols)
 
     def _reflect_fk(
-            self, table_name, schema, table, cols_by_orig_name,
-            exclude_columns, reflection_options):
+        self,
+        table_name,
+        schema,
+        table,
+        cols_by_orig_name,
+        exclude_columns,
+        resolve_fks,
+        _extend_on,
+        reflection_options,
+    ):
         fkeys = self.get_foreign_keys(
-            table_name, schema, **table.dialect_kwargs)
+            table_name, schema, **table.dialect_kwargs
+        )
         for fkey_d in fkeys:
-            conname = fkey_d['name']
+            conname = fkey_d["name"]
             # look for columns by orig name in cols_by_orig_name,
             # but support columns that are in-Python only as fallback
             constrained_columns = [
-                cols_by_orig_name[c].key
-                if c in cols_by_orig_name else c
-                for c in fkey_d['constrained_columns']
+                cols_by_orig_name[c].key if c in cols_by_orig_name else c
+                for c in fkey_d["constrained_columns"]
             ]
             if exclude_columns and set(constrained_columns).intersection(
-                    exclude_columns):
+                exclude_columns
+            ):
                 continue
-            referred_schema = fkey_d['referred_schema']
-            referred_table = fkey_d['referred_table']
-            referred_columns = fkey_d['referred_columns']
+            referred_schema = fkey_d["referred_schema"]
+            referred_table = fkey_d["referred_table"]
+            referred_columns = fkey_d["referred_columns"]
             refspec = []
             if referred_schema is not None:
-                sa_schema.Table(referred_table, table.metadata,
-                                autoload=True, schema=referred_schema,
-                                autoload_with=self.bind,
-                                **reflection_options
-                                )
+                if resolve_fks:
+                    sa_schema.Table(
+                        referred_table,
+                        table.metadata,
+                        autoload=True,
+                        schema=referred_schema,
+                        autoload_with=self.bind,
+                        _extend_on=_extend_on,
+                        **reflection_options
+                    )
                 for column in referred_columns:
-                    refspec.append(".".join(
-                        [referred_schema, referred_table, column]))
+                    refspec.append(
+                        ".".join([referred_schema, referred_table, column])
+                    )
             else:
-                sa_schema.Table(referred_table, table.metadata, autoload=True,
-                                autoload_with=self.bind,
-                                schema=sa_schema.BLANK_SCHEMA,
-                                **reflection_options
-                                )
+                if resolve_fks:
+                    sa_schema.Table(
+                        referred_table,
+                        table.metadata,
+                        autoload=True,
+                        autoload_with=self.bind,
+                        schema=sa_schema.BLANK_SCHEMA,
+                        _extend_on=_extend_on,
+                        **reflection_options
+                    )
                 for column in referred_columns:
                     refspec.append(".".join([referred_table, column]))
-            if 'options' in fkey_d:
-                options = fkey_d['options']
+            if "options" in fkey_d:
+                options = fkey_d["options"]
             else:
                 options = {}
             table.append_constraint(
-                sa_schema.ForeignKeyConstraint(constrained_columns, refspec,
-                                               conname, link_to_name=True,
-                                               **options))
+                sa_schema.ForeignKeyConstraint(
+                    constrained_columns,
+                    refspec,
+                    conname,
+                    link_to_name=True,
+                    **options
+                )
+            )
+
+    _index_sort_exprs = [
+        ("asc", operators.asc_op),
+        ("desc", operators.desc_op),
+        ("nullsfirst", operators.nullsfirst_op),
+        ("nullslast", operators.nullslast_op),
+    ]
 
     def _reflect_indexes(
-        self, table_name, schema, table, cols_by_orig_name,
-            include_columns, exclude_columns, reflection_options):
+        self,
+        table_name,
+        schema,
+        table,
+        cols_by_orig_name,
+        include_columns,
+        exclude_columns,
+        reflection_options,
+    ):
         # Indexes
         indexes = self.get_indexes(table_name, schema)
         for index_d in indexes:
-            name = index_d['name']
-            columns = index_d['column_names']
-            unique = index_d['unique']
-            flavor = index_d.get('type', 'index')
-            dialect_options = index_d.get('dialect_options', {})
+            name = index_d["name"]
+            columns = index_d["column_names"]
+            column_sorting = index_d.get("column_sorting", {})
+            unique = index_d["unique"]
+            flavor = index_d.get("type", "index")
+            dialect_options = index_d.get("dialect_options", {})
 
-            duplicates = index_d.get('duplicates_constraint')
-            if include_columns and \
-                    not set(columns).issubset(include_columns):
+            duplicates = index_d.get("duplicates_constraint")
+            if include_columns and not set(columns).issubset(include_columns):
                 util.warn(
-                    "Omitting %s key for (%s), key covers omitted columns." %
-                    (flavor, ', '.join(columns)))
+                    "Omitting %s key for (%s), key covers omitted columns."
+                    % (flavor, ", ".join(columns))
+                )
                 continue
             if duplicates:
                 continue
@@ -763,25 +904,40 @@ class Inspector(object):
             idx_cols = []
             for c in columns:
                 try:
-                    idx_col = cols_by_orig_name[c] \
-                        if c in cols_by_orig_name else table.c[c]
+                    idx_col = (
+                        cols_by_orig_name[c]
+                        if c in cols_by_orig_name
+                        else table.c[c]
+                    )
                 except KeyError:
                     util.warn(
                         "%s key '%s' was not located in "
-                        "columns for table '%s'" % (
-                            flavor, c, table_name
-                        ))
-                else:
-                    idx_cols.append(idx_col)
+                        "columns for table '%s'" % (flavor, c, table_name)
+                    )
+                    continue
+                c_sorting = column_sorting.get(c, ())
+                for k, op in self._index_sort_exprs:
+                    if k in c_sorting:
+                        idx_col = op(idx_col)
+                idx_cols.append(idx_col)
 
             sa_schema.Index(
-                name, *idx_cols,
-                **dict(list(dialect_options.items()) + [('unique', unique)])
+                name,
+                *idx_cols,
+                _table=table,
+                **dict(list(dialect_options.items()) + [("unique", unique)])
             )
 
     def _reflect_unique_constraints(
-        self, table_name, schema, table, cols_by_orig_name,
-            include_columns, exclude_columns, reflection_options):
+        self,
+        table_name,
+        schema,
+        table,
+        cols_by_orig_name,
+        include_columns,
+        exclude_columns,
+        reflection_options,
+    ):
 
         # Unique Constraints
         try:
@@ -791,15 +947,14 @@ class Inspector(object):
             return
 
         for const_d in constraints:
-            conname = const_d['name']
-            columns = const_d['column_names']
-            duplicates = const_d.get('duplicates_index')
-            if include_columns and \
-                    not set(columns).issubset(include_columns):
+            conname = const_d["name"]
+            columns = const_d["column_names"]
+            duplicates = const_d.get("duplicates_index")
+            if include_columns and not set(columns).issubset(include_columns):
                 util.warn(
                     "Omitting unique constraint key for (%s), "
-                    "key covers omitted columns." %
-                    ', '.join(columns))
+                    "key covers omitted columns." % ", ".join(columns)
+                )
                 continue
             if duplicates:
                 continue
@@ -808,20 +963,32 @@ class Inspector(object):
             constrained_cols = []
             for c in columns:
                 try:
-                    constrained_col = cols_by_orig_name[c] \
-                        if c in cols_by_orig_name else table.c[c]
+                    constrained_col = (
+                        cols_by_orig_name[c]
+                        if c in cols_by_orig_name
+                        else table.c[c]
+                    )
                 except KeyError:
                     util.warn(
                         "unique constraint key '%s' was not located in "
-                        "columns for table '%s'" % (c, table_name))
+                        "columns for table '%s'" % (c, table_name)
+                    )
                 else:
                     constrained_cols.append(constrained_col)
             table.append_constraint(
-                sa_schema.UniqueConstraint(*constrained_cols, name=conname))
+                sa_schema.UniqueConstraint(*constrained_cols, name=conname)
+            )
 
     def _reflect_check_constraints(
-            self, table_name, schema, table, cols_by_orig_name,
-            include_columns, exclude_columns, reflection_options):
+        self,
+        table_name,
+        schema,
+        table,
+        cols_by_orig_name,
+        include_columns,
+        exclude_columns,
+        reflection_options,
+    ):
         try:
             constraints = self.get_check_constraints(table_name, schema)
         except NotImplementedError:
@@ -829,5 +996,14 @@ class Inspector(object):
             return
 
         for const_d in constraints:
-            table.append_constraint(
-                sa_schema.CheckConstraint(**const_d))
+            table.append_constraint(sa_schema.CheckConstraint(**const_d))
+
+    def _reflect_table_comment(
+        self, table_name, schema, table, reflection_options
+    ):
+        try:
+            comment_dict = self.get_table_comment(table_name, schema)
+        except NotImplementedError:
+            return
+        else:
+            table.comment = comment_dict.get("text", None)
